
# Typage de C--

## Types déclarés

```ocaml
(* types.mli *)
type typ = Tscalar of typ_basic | Tfun of typ_fun | Tstring | Tnull

(* type entier ou pointeur de ... d'entier *)
and typ_basic = Tint | Tptr of typ_basic

(* type pointeur de fonction *)
and typ_fun = { result : typ; args : typ list }

(* ctyping.ml *)
type typ_var =
  | TVvar of typ (* Une variable *)
  | TVfun of typ_fun (* Une fonction globale *)

(* Type pour stocker les déclarations courantes *)
type declaration_tbl = (string, typ_var) Hashtbl.t
```

Les déclarations de fonctions / variables seront stockées dans une `Hashtbl` qui à son nom associe
son typ_var.

Une fonction seras représentée par `TVfun of typ_fun` où typ_fun est sa signature.

Alors qu'une variable seras représentée par `TVvar of typ`. Si typ est de `typ_fu` cela veut dire
que la variables est un pointeur de fonction.

J'ai décidé d'ajouter NULL au parser pour pouvoir affecter et comparer des pointeurs.
Son type est `Tnull` ce qui représente un pointeur "polymorphique", c'est à dire que c'est 
un `Tptr *`. Il ne permet pas de représenter un pointeur de fonction.

## Fonctionnement du code

### main.ml

Il appelle `ctyping.check_file` et renvoie le code de sortie 

  - 0 si tout s'est bien passé
  - 1 en cas d'erreur du lexeur 
  - 2 en cas d'erreur de syntaxe 
  - 3 en cas d'erreur de typage
  - 4 en cas d'anomalie du typeur 
  - 5 en cas d'anomalie 
 
### ctyping.ml

Il s'occupe du typage.

Une majorité de fonction prennent `(decs : declaration_tbl)` en argument qui représente les 
déclarations de variables et de fonctions déjà faites.

La fonction `type_loc_code (decs : declaration_tbl) (ret : typ) ((l, c) : loc_code) : bool` 
s'occupe de typer la commande `(l, c)` et de renvoyer si toutes les branches renvoient le bon type.

La fonction `type_expr (decs : declaration_tbl) ((l, e) : loc_expr) : typ` s'occupe de typer une
expression et de renvoyer son type.

Comme on a un type `Tnull` qui représente un pointeur quelconque, on a besoin d'une fonction
`cmp_types (a : typ) (b : typ) : typ option` qui vérifie si deux types sont égaux par exemple
 - `cmp_types Tint Tnull` renvoie `None`
 - `cmp_types (Tfun f) Tnull` renvoie `Some f`
 - `cmp_types (Tptr (Tptr Tint)) Tnull` renvoie `Some (Tptr (Tpr Tint))`.

J'ai considéré que les opérateurs `&` ne marche qu'avec une variable (et non une expression 
arbitraire) de type scalaire (`typ_basic`).

J'ai considéré que `++`, `--` pouvait être utilisé avec soit une variable soit un déférencement d'un
pointeur': `++x`, `(*x)++`, `(*(x + 15)++)` sont valides si le type de x convient.

## Tests

La commande make test permet de lancer tous mes tests du dossier tst.

Les fichiers des dossiers err-x attendent le code d'erreur x alors que ceux du dossier ok sont bons.
