# Remarques

Le code à été testé avec LC3Tools (qui entre autre permet de faire des .FILL label) et avec OCaml 5.3.

Les opérations sont signés sur 16-bits, néanmoins les opérations fonctionne mal avec -2^15 (car il
est son propre inverse). De plus, je ne suis pas sûr que ma division se comporte très bien en cas
d'overflow.

Le code implémente un nombre arbitraire de variables locales et paramètres (dans la limite de la
taille de la pile), mais ils sont implémentés de manière naïve, avec des répétitions de ADD pour
faire un LDR avec un offset plus grand que 32.

Si l'assembleur fait des sauts trop grands (pour une instruction BR / JSR), on peut utiliser
l'option --long-jmp pour ccomp qui transforme les BR en des BR longue portée (en chargeant un label
pas très loin).

Pour pallier le fait que l'on a seulement un nombre fini de registre, j'alloue sur la pile une place
pour les registres (ils se comportent comme des variables locales). À la différence qu'ils sont
accédés par un offset depuis R6 et non depuis R4 comme les variables locales / paramètres (cf schéma
ci dessous).

# Registres

R7: adresse de retour
R6: pointeur de bas de pile
R5: pointeur vers les constantes 
R4: bas de cadre
R0,R1,(R2),(R3): registres de travail
R0: registre de retour des fonctions

Cadre d'un appel de fonction

…
Sauvegarde de R4
Sauvegarde de R7
A0 <- R4
Args …
An
L0
Locals …
Lk
reg_0
Registres_cfg
reg_p <- R6


# Structure de l'assembleur

L'assembleur est généré sous cette forme :

--
Section d'initialisation
--
Fonction a
Fonction b
…
--
Fonction CMP
Fonction MULT
Fonction DIV
Fonction PrintInt
--
Variables globales
--
Chaînes de caractères
--

Pour optimiser la compacité des variables globales, seulement l'adresse des chaînes de caractère est
stockée (utilisation d'un .FILL STR_LABEL).

# Architecture du code

La synthèse se fait en deux passes, tout d'abord on transforme le cfg en `asm_ins list` : on
transforme les registre du cfg en position dans la pile, on alloue un identifiant pour chaque
globale… Ensuite, lors de la deuxième phase, on transforme ce 'semi-cfg' en instructions
assembleurs.

Il y a 2 systèmes d'affichage, les `pp_asm_*` servent à debugger le 'semi-cfg' alors que
les `pp_synth_*` servent à afficher le 'semi-cfg' sous forme d'instructions assembleurs.

# Tests

Pour lancer les tests de synthèse, il faut télécharger le code-source de LC3Tools, ajouter le tests
`exec.cpp` dans `src/test/tests/samples` :

```cpp
#define API_VER 2
#include "framework.h"

void testBringup(lc3::sim & sim) {
    sim.writePC(0x3000);
    sim.setRunInstLimit(500000);
 }

void testTeardown(lc3::sim & sim) {
    (void) sim;
}

void TestExec(lc3::sim & sim, Tester & tester, double total_points) {
    sim.run();
    tester.verify("Exit", ! sim.didExceedInstLimit(), total_points);
}

void setup(Tester & tester) {
        tester.registerTest("Exec", TestExec, 10, true);
 }

void shutdown(void) { }
```

Ensuite, il faut compiler le code de LC3Tools, et copier le fichier lc3tools/build/bin/exec dans le
dossier bin du compilateur. Ce fichier sert à exécuter le code assembleur LC3 pour vérifier la
correction des tests.
