#!/bin/bash
cd ../
CFGFLAGS=--cfg-simple

ERRS_FILES=

for file in tst/ok/*.c
do
    echo -e "[\e[35mBEG\e[0m] \e[34m$file\e[0m"
    ./src/ccomp --analyse-only $file 2> ${file%.c}.log
    if [ $? -eq 0 ]
    then
        echo -e "[\e[32mOK \e[0m] \e[34m${file%.c}.log\e[0m"
    else
        cat ${file%.c}.log
        echo -e "[\e[31mERR\e[0m] \e[34m${file%.c}.log\e[0m"
        ERRS_FILES="$ERRS_FILES\n - $file"
    fi
done

for err_code in 2 3
do
    for file in tst/err-$err_code/*.c
    do
        echo -e "[\e[35mBEG\e[0m] \e[34m$file\e[0m"
        ./src/ccomp --analyse-only $file 2> ${file%.c}.log
        if [ $? -eq $err_code ]
        then
            echo -e "[\e[32mOK \e[0m] \e[34m${file%.c}.log\e[0m"
        else
            cat ${file%.c}.log
            echo -e "[\e[31mERR\e[0m] \e[34m${file%.c}.log\e[0m"
            ERRS_FILES="$ERRS_FILES\n - $file"
        fi
    done
done

for file in tst/cfg/*.c
do
    echo -e "[\e[35mBEG\e[0m] \e[34m$file\e[0m"
    ./src/ccomp --analyse-only ${CFGFLAGS} --analyse-no-cf --analyse-no-filter --convert-cfg ${file%.c}.cfg.log $file 2> ${file%.c}.log
    if [ $? -eq 0 ] && cmp -s ${file%.c}.cfg.log ${file%.c}.cfg
    then
        echo -e "[\e[32mOK \e[0m] \e[34m${file%.c}.log\e[0m"
    else
        diff -u ${file%.c}.cfg.log ${file%.c}.cfg >> ${file%.c}.log
        cat ${file%.c}.log
        echo -e "[\e[31mERR\e[0m] \e[34m${file%.c}.log\e[0m"
        ERRS_FILES="$ERRS_FILES\n - $file"
    fi
done

for file in tst/analysis/*.c
do
    echo -e "[\e[35mBEG\e[0m] \e[34m$file\e[0m"
    ./src/ccomp --analyse-only ${CFGFLAGS} --convert-cfg ${file%.c}.cfg.log $file 2> ${file%.c}.log
    if [ $? -eq 0 ] && (grep "warning:" ${file%.c}.log | cmp -s - ${file%.c}.out) &&
        cmp -s ${file%.c}.cfg.log ${file%.c}.cfg
    then
        echo -e "[\e[32mOK \e[0m] \e[34m${file%.c}.log\e[0m"
    else
        cat ${file%.c}.log
        diff -u ${file%.c}.cfg.log ${file%.c}.cfg >> ${file%.c}.log
        echo -e "[\e[31mERR\e[0m] \e[34m${file%.c}.log\e[0m"
        ERRS_FILES="$ERRS_FILES\n - $file"
    fi
done

for file in tst/synth/*.c
do
    echo -e "[\e[35mBEG\e[0m] \e[34m$file\e[0m"
    ./src/ccomp --long-jmp $file > ${file%.c}.asm.log 2> ${file%.c}.log
    if [ $? -eq 0 ] && (./bin/exec --asm-print-level=3 --print-output ${file%.c}.asm.log 2> ${file%.c}.exec.log >> ${file%.c}.log && cmp -s ${file%.c}.exec.log ${file%.c}.out)
    then
        echo -e "[\e[32mOK \e[0m] \e[34m${file%.c}.log\e[0m"
    else
        cat ${file%.c}.log
        diff -u ${file%.c}.exec.log ${file%.c}.out >> ${file%.c}.log
        echo -e "[\e[31mERR\e[0m] \e[34m${file%.c}.log\e[0m"
        ERRS_FILES="$ERRS_FILES\n - $file"
    fi
done

if [ -n "$ERRS_FILES" ]
then
    echo -e "\n\e[31mErreurs :\e[0m"
    echo -e $ERRS_FILES
else
    echo -e "\n\e[32mSuccés !\e[0m"
fi